import os
import supervisely as sly
import cv2

# in this directory stored 7 copy of one video with different frame rotations
videos_dir = "./rotated"

new_videos_dir = "./correct_orientation"
fourcc = cv2.VideoWriter_fourcc(*"MP4V")

if sly.fs.dir_exists(new_videos_dir):
    sly.fs.clean_dir(new_videos_dir)
else:
    sly.fs.mkdir(new_videos_dir)

frames_set = set()  # for test at the end

for file in os.listdir(videos_dir):
    new_path = os.path.join(new_videos_dir, f"new-{file}")
    filepath = os.path.join(videos_dir, file)

    cap = cv2.VideoCapture(filepath)

    # set CAP_PROP_ORIENTATION_AUTO flag for VideoCapture
    cap.set(cv2.CAP_PROP_ORIENTATION_AUTO, 1)

    frameSize = (int(cap.get(3)), int(cap.get(4)))
    fps = cap.get(5)

    frames_set.add(frameSize)  # for test at the end

    writer = cv2.VideoWriter(new_path, fourcc, fps, frameSize)
    while cap.isOpened():
        success, frame = cap.read()
        if not success:
            print("Can't receive frame (stream end?). Exiting ...")
            break
        writer.write(frame)

    # Release everything if job is finished
    writer.release()
    cap.release()
    cv2.destroyAllWindows()

# should be only 2 shape sizes options: 1280x720 or 720x1280
assert len(frames_set) == 2



# <div v-if="data.progress.message" class="mt10">
#     <div style="color: #20a0ff">
#         {{data.progress.message}}: {{data.progress.current}} / {{data.progress.total}}
#     </div>
#     <el-progress :percentage="data.progress.percent"></el-progress>
# </div>

import math
from supervisely.api.api import Api
from supervisely.task.progress import Progress


class ProgressBar:
    def __init__(self, task_id, api: Api, v_model, message, total=None, is_size=False, min_report_percent=1):
        self._task_id = task_id
        self._api = api
        self._v_model = v_model
        if self._v_model.startswith("data.") is False:
            new_v_model = v_model.replace('state.', 'data.', 1)
            raise KeyError(f"Data for this widget has to be stored in data field. "
                           f"Please, change value of v_model argument "
                           f"from {v_model} to {new_v_model} manually and also check html template")

        self._message = message
        self._is_size = is_size
        self._min_report_percent = min_report_percent
        self._total = None
        self._progress: Progress = None
        self.set_total(total)

    def get_total(self):
        return self._total

    def set_total(self, total):
        self._total = total
        if total is not None:
            self._progress = Progress(
                self._message,
                self._total,
                is_size=self._is_size,
                min_report_percent=self._min_report_percent
            )
        else:
            self._progress = None

    def init_data(self, data: dict):
        data[self._v_model.split(".")[1]] = self.to_json()

    def to_json(self):
        res = {
            "message": self._progress.message if self._progress is not None else None,
            "current": self._progress.current_label if self._progress is not None else None,
            "total": self._progress.total_label if self._progress is not None else None,
            "percent": math.floor(self._progress.current * 100 / self._progress.total) if self._progress is not None else None,
        }
        return res

    def get_field(self):
        return {"field": self._v_model, "payload": self.to_json()}

    def reset(self):
        self._progress = None

    def reset_and_update(self):
        self.reset()
        self.update(force=True)

    def is_reset(self):
        return self._progress is None

    def update(self, force=False):
        if self._progress is not None and (self._progress.need_report() or force):
            self._progress.report_progress()
            self._api.app.set_fields(self._task_id, [self.get_field()])
        if self._progress is None:
            self._api.app.set_fields(self._task_id, [self.get_field()])

    def set(self, value, force_update=False):
        if self._progress is None:
            self.set_total(self._total)
        self._progress.set_current_value(value)
        self.update(force_update)

    def increment(self, count, force_update=False):
        if self._progress is None:
            self.set_total(self._total)
        self._progress.iters_done(count)
        self.update(force_update)



from __future__ import annotations
from typing import List, Dict, Optional, Any, Iterator
from supervisely.pointcloud_annotation.pointcloud_tag_collection import PointcloudTagCollection
from supervisely.pointcloud_annotation.pointcloud_episode_tag import PointcloudEpisodeTag
from supervisely.annotation.tag_meta_collection import TagMetaCollection
from supervisely.annotation.tag_meta import TagMeta
from supervisely.video_annotation.key_id_map import KeyIdMap

class PointcloudEpisodeTagCollection(PointcloudTagCollection):
    item_type = PointcloudEpisodeTag

    def __iter__(self) -> Iterator[PointcloudEpisodeTag]:
        return next(self)

    @classmethod
    def from_api_response(
        cls, 
        data: List[Dict], 
        tag_meta_collection: TagMetaCollection, 
        id_to_tagmeta: Optional[Dict[int, TagMeta]] = None
    ) -> PointcloudEpisodeTagCollection:
        return super().from_api_response(data, tag_meta_collection, id_to_tagmeta=id_to_tagmeta)

    def get_by_name(self, tag_name: str) -> List[PointcloudEpisodeTag]:
        return super().get_by_name(tag_name)

    def get_single_by_name(self, tag_name: str) -> PointcloudEpisodeTag:
        return super().get_single_by_name(tag_name)

    @classmethod
    def from_json(
        cls, 
        data: List[Dict], 
        tag_meta_collection: TagMetaCollection, 
        key_id_map: Optional[KeyIdMap] = None,
    ) -> PointcloudEpisodeTagCollection:
        return super().from_json(data, tag_meta_collection, key_id_map=key_id_map)


# coding: utf-8

# docs
from typing import List, Optional

from supervisely.video_annotation.key_id_map import KeyIdMap
from supervisely.video_annotation.video_object_collection import VideoObjectCollection
from supervisely.api.entity_annotation.object_api import ObjectApi


class VideoObjectApi(ObjectApi):
    def append_bulk(self, video_id: int, objects: VideoObjectCollection, key_id_map: Optional[KeyIdMap]=None) -> List[int]:
        info = self._api.video.get_info_by_id(video_id)
        return self._append_bulk(self._api.video.tag, video_id, info.project_id, info.dataset_id, objects, key_id_map)



from typing import Union
from supervisely.project.project_meta import ProjectMeta
from supervisely.api.api import Api
from supervisely.annotation.annotation import Annotation


# html example
# <sly-grid-gallery
#         v-if="data.gallery"
#         :content="data.gallery.content"
#         :options="data.gallery.options">
#     <template v-slot:card-footer="{ annotation }">
#         <div class="mt5" style="text-align: center">
#             <el-tag type="primary">{{annotation.title}}</el-tag>
#         </div>
#     </template>
# </sly-grid-gallery>


class CompareGallery:

    def __init__(self, task_id, api: Api, v_model, project_meta: ProjectMeta):
        self._task_id = task_id
        self._api = api
        self._v_model = v_model
        self._project_meta = project_meta.clone()

        self._left_title = None
        self._left_image_url = None
        self._left_ann = None

        self._right_title = None
        self._right_image_url = None
        self._right_ann = None

        self._options = {
            "enableZoom": True,
            "syncViews": True,
            "showPreview": False,
            "selectable": False,
            "opacity": 0.5,
            "showOpacityInHeader": True,
            # "viewHeight": 450,
        }
        self._options_initialized = False

    def update_project_meta(self, project_meta: ProjectMeta):
        self._project_meta = project_meta.clone()

    def _set_item(self, name, title, image_url, ann: Union[Annotation, dict] = None):
        setattr(self, f"_{name}_title", title)
        setattr(self, f"_{name}_image_url", image_url)
        res_ann = Annotation((1,1))
        if ann is not None:
            if type(ann) is dict:
                res_ann = Annotation.from_json(ann, self._project_meta)
            else:
                res_ann = ann.clone()
        setattr(self, f"_{name}_ann", res_ann)

    def set_left(self, title, image_url, ann: Union[Annotation, dict] = None):
        self._set_item("left", title, image_url, ann)

    def set_right(self, title, image_url, ann: Union[Annotation, dict] = None):
        self._set_item("right", title, image_url, ann)

    def _get_item_annotation(self, name):
        return {
            "url": getattr(self, f"_{name}_image_url"),
            "figures": [label.to_json() for label in getattr(self, f"_{name}_ann").labels],
            "title": getattr(self, f"_{name}_title"),
        }

    def update(self, options=True):
        if self._left_image_url is None:
            raise ValueError("Left item is not defined")
        if self._right_image_url is None:
            raise ValueError("Right item is not defined")
        gallery_json = self.to_json()
        if options is True or self._options_initialized is False:
            self._api.task.set_field(self._task_id, self._v_model, gallery_json)
            self._options_initialized = True
        else:
            self._api.task.set_field(self._task_id, f"{self._v_model}.content", gallery_json["content"])

    def to_json(self):
        if self._left_image_url is None or self._right_image_url is None:
            return None

        return {
            "content": {
                "projectMeta": self._project_meta.to_json(),
                "layout": [["left"], ["right"]],
                "annotations": {
                    "left": self._get_item_annotation("left"),
                    "right": self._get_item_annotation("right"),
                }
            },
            "options": {
                **self._options,
                "syncViewsBindings": [["left", "right"]]
            }
        }
